@@ -1,17 +1,18 @@
-#!/usr/bin/python2
+#!/usr/bin/python3
 
 import os
-import commands
 import gettext
 import apt_pkg
 import subprocess
 import tempfile
 import locale
+import codecs
 
 import gi
 gi.require_version('Gtk', '3.0')
+gi.require_version('XApp', '1.0')
 from gi.repository import GdkX11
-from gi.repository import Gtk, GdkPixbuf
+from gi.repository import Gtk, GdkPixbuf, XApp
 
 # i18n
 APP = 'mintlocale'
@@ -42,7 +43,7 @@ class MintLocale:
         self.selected_language_packs = None
 
         self.language_packs = []
-        with open("/usr/share/linuxmint/mintlocale/language_packs") as f:
+        with codecs.open("/usr/share/linuxmint/mintlocale/language_packs", 'r', encoding='utf-8') as f:
             for line in f:
                 line = line.strip()
                 columns = line.split(":")
@@ -58,11 +59,12 @@ class MintLocale:
         self.cache = apt_pkg.Cache(None)
         self.cache_updated = False
 
+
         self.builder = Gtk.Builder()
         self.builder.set_translation_domain("mintlocale")
         self.builder.add_from_file('/usr/share/linuxmint/mintlocale/install_remove.ui')
         self.window = self.builder.get_object("main_window")
-
+        self.window.set_icon_name("preferences-desktop-locale")
         self.builder.get_object("main_window").connect("destroy", Gtk.main_quit)
 
         self.treeview = self.builder.get_object("treeview_language_list")
@@ -135,27 +137,27 @@ class MintLocale:
 
         # Load countries into memory
         self.countries = {}
-        file = open('/usr/share/linuxmint/mintlocale/countries', "r")
-        for line in file:
-            line = line.strip()
-            split = line.split("=")
-            if len(split) == 2:
-                self.countries[split[0]] = split[1]
-        file.close()
+        with codecs.open('/usr/share/linuxmint/mintlocale/countries', "r", encoding="utf-8") as file:
+            for line in file:
+                line = line.strip()
+                split = line.split("=")
+                if len(split) == 2:
+                    self.countries[split[0]] = split[1]
 
         # Load languages into memory
         self.languages = {}
-        file = open('/usr/share/linuxmint/mintlocale/languages', "r")
-        for line in file:
-            line = line.strip()
-            split = line.split("=")
-            if len(split) == 2:
-                self.languages[split[0]] = split[1]
-        file.close()
+        with codecs.open('/usr/share/linuxmint/mintlocale/languages', "r", encoding='utf-8') as file:
+            for line in file:
+                line = line.strip()
+                split = line.split("=")
+                if len(split) == 2:
+                    self.languages[split[0]] = split[1]
 
-        locales = commands.getoutput("localedef --list-archive")
+        locales = subprocess.check_output("localedef --list-archive", shell=True)
+        locales = locales.decode('utf-8').strip()
         for line in locales.split("\n"):
-            line = line.replace("utf8", "UTF-8")
+            line = line.replace("utf8", "UTF-8").strip()
+
             locale_code = line.split(".")[0].strip()
             charmap = None
             if len(line.split(".")) > 1:
@@ -260,7 +262,8 @@ class MintLocale:
         os.system("localedef --delete-from-archive %s" % locale)
         # If there are no more locales using the language, remove the language packs
         (language_code, country_code, language_label) = self.split_locale(locale)
-        num_locales = commands.getoutput("localedef --list-archive | grep %s_ | wc -l" % language_code)
+        num_locales = subprocess.check_output("localedef --list-archive | grep %s_ | wc -l" % language_code, shell=True)
+        num_locales = num_locales.decode('utf-8').strip()
         # Check if the language packs are installed
         if num_locales == "0":
             installed_packs = []
@@ -270,16 +273,10 @@ class MintLocale:
                         pkg = self.cache[pkgname]
                         if (pkg.has_versions and pkg.current_state == apt_pkg.CURSTATE_INSTALLED):
                             installed_packs.append(pkg)
-                            print pkg
+                            print(pkg)
 
             if len(installed_packs) > 0:
-                cmd = ["/usr/sbin/synaptic", "--hide-main-window", "--non-interactive", "--parent-window-id", "%s" % self.builder.get_object("main_window").get_window().get_xid()]
-                cmd.append("-o")
-                cmd.append("Synaptic::closeZvt=true")
-                cmd.append("--progress-str")
-                cmd.append("\"" + _("Please wait, this can take some time") + "\"")
-                cmd.append("--finish-str")
-                cmd.append("\"" + _("The related language packs were removed") + "\"")
+                cmd = ["/usr/sbin/synaptic", "--hide-main-window", "--non-interactive", "--parent-window-id", "%s" % self.builder.get_object("main_window").get_window().get_xid(), "-o", "Synaptic::closeZvt=true"]
                 f = tempfile.NamedTemporaryFile()
                 for pkg in installed_packs:
                     f.write("%s\tdeinstall\n" % pkg.name)
